/**
	This creates a temporary file, deletes it and keeps using the file 
	descriptor returned by the open syscall.

	You can access the file contents via
	cat /proc/<PID>/fd/3
	This of course requires superuser privileges.

	You can get the pid of the process with
	ps aux | grep hiddenfile
	where hiddenfile is the name of this binary. The PID can be found in
	the second column.
*/

#include <time.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

int main(int argc, char** argv) {

	char* fname;
	int fd, ret;

	// generate unique random filename in /tmp
	fname = tempnam("/tmp", NULL);
	printf("file %s created\n", fname);
	if (fname == NULL) {
		perror("Could not create random temp file name");
		exit(EXIT_FAILURE);		
	}

	// create/open fname
	fd = open(fname, O_RDWR | O_CREAT);
	if (fd == -1) {
		perror("Could not open temp file");
		exit(EXIT_FAILURE);
	}

	// unlink that file
	ret = unlink(fname);
	if (ret != 0) {
		perror("Could not unlink temp file"); 
		exit(EXIT_FAILURE);
	}

	// write some stuff to the hidden file until we are aborted,
	// so that we can try to read the "secrets"

	printf("Kill with Ctrl+C, once you read my secret file (%s)!\n", fname);

	while (1) {
		int buflen;
		time_t timer;
		int buffer_len;
		int written = 0;
		char timestr[32];
		char buffer[128];
		struct tm* tm_info;

		// get the local time together with a small message into buffer
		time(&timer);
		tm_info = localtime(&timer);
		strftime(timestr, 32, "%Y:%m:%d %H:%M:%S", tm_info);
		snprintf(buffer, 128, "Hello World! The time is now: %s\n", timestr);
		buffer_len = strlen(buffer);

		// write it to our hidden file
		do {
			written += write(fd, buffer + written, buffer_len - written);
		} while (written != buffer_len);

		// make sure the file contents are flushed to disk
		syncfs(fd);

		sleep(5);
	}

	close(fd);
	free(fname);

}

