/**
	This should give a simillar output as executing "ls -l" with coloring
	turned off. Unlike ls we do not sort our output.

	There are many things which are not handled in this code, i.e., sticky and
	suid bits, symlinks.
*/

#include <pwd.h>
#include <grp.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>

void display_entry(char *name, struct stat *s) {
	int i;
	char perms[11];
	char mtime[32];
	char type;
	struct passwd *pwuid;
	struct group *grgid;
	struct tm *tm;

	if (name[0] == '.') return; //ignore files starting with a dot

	pwuid = getpwuid(s->st_uid); //get info about the owner user id
	grgid = getgrgid(s->st_gid); //get into about the owner group id

	// create string representation of type and permissions
	memset(perms, '-', 10);
	perms[10] = '\0';

	// Determine file type
	type = '-';
	switch (s->st_mode & S_IFMT) {
	case S_IFBLK:
		type = 'b';
		break;
	case S_IFCHR:
		type = 'c';
		break;
	case S_IFDIR:
		type = 'd';
		break;
	case S_IFIFO:
		type = 'p';
		break;
	case S_IFLNK:
		type = 'l';
		break;
	case S_IFSOCK:
		type = 's';
		break;
	}
	perms[0] = type;

	// Get permissions
	if (s->st_mode & S_IRUSR) perms[1] = 'r';
	if (s->st_mode & S_IWUSR) perms[2] = 'w';
	if (s->st_mode & S_IXUSR) perms[3] = 'x';
	if (s->st_mode & S_IRGRP) perms[4] = 'r';
	if (s->st_mode & S_IWGRP) perms[5] = 'w';
	if (s->st_mode & S_IXGRP) perms[6] = 'x';
	if (s->st_mode & S_IROTH) perms[7] = 'r';
	if (s->st_mode & S_IWOTH) perms[8] = 'w';
	if (s->st_mode & S_IXOTH) perms[9] = 'x';

	// create string representation of modification time
	tm = localtime(&(s->st_mtim.tv_sec));
	strftime(mtime, 32, "%Y-%m-%d %H:%M:%S", tm);

	printf("%s %i %s %s %lu %s %s\n",
	       perms, s->st_nlink, pwuid->pw_name, grgid->gr_name, s->st_size,
		   mtime, name);
}

int main(void) {
	DIR *dir;
	struct stat statbuf;
	struct dirent *dentry;

	// open the current directory
	dir = opendir(".");
	if (dir == NULL) {
		perror("Error in opendir()");
		exit(EXIT_FAILURE);
	}

	// iterate through it
	while ((dentry = readdir(dir)) != NULL) {
		lstat(dentry->d_name, &statbuf);
		display_entry(dentry->d_name, &statbuf);
	}

	exit(EXIT_SUCCESS);
}
